
% REPRODUCING EXPERIMENT 1

clear; 


% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scale = .7;  % scale of Gabors, length = 17 pixels
norient = 15;  % number of orientations
Upperbound = 16.; % saturation value
epsilon = .1;  % allowed correlation between selected Gabors
Totalsketch = 40;  % allowed number of selected Gabors   
sub = 2;    % subsampling rate for Gabor
Lrange = 3; % allowed shifting in location (times sub pixels)
Orange = 1; % allowed shifting in orientation
%%% Note: Lrange*sub is the pixels allowed to shift along normal
%%%       Lrange*sub should be the same in learning and testing

SHUTUP = 0.; % SHUTUP is to neglect the positions with low average responses          

fold = 1.; 
Folder = 'mixedTrain2';
Iname = dir([Folder '\*.jpg']);
nimage = size(Iname, 1);
Isize = zeros(nimage, 2);
for i = 1 : nimage
    tmpIm = imread([Folder '\' Iname(i).name]);
    if size(tmpIm,3) == 3
        tmpIm = rgb2gray(tmpIm);
    end
    I{i} = double(tmpIm); 
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for (i = 1 : nimage)
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the training images the same



% FILTERING IMAGES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(['start filtering']);
tic
[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
C = corr(allfilter, epsilon);    % generate the inhibition maps
allfiltered = applyfilterfftsame(I, allfilter);  % filter training images
allfilteredback = mcopy(allfiltered, nimage, norient); 

disp(['filtering time: ' num2str(toc) ' seconds']);




% VARIABLES FOR MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mi = zeros(1, Totalsketch);  % orientation of selected Gabor
Mx = zeros(1, Totalsketch);  % position of selected Gabor
My = zeros(1, Totalsketch); 
Mm = zeros(1, Totalsketch);  % lambda
Mm1 = zeros(1, Totalsketch);  % logZ
sym = zeros(sx, sy);  % symbolic plot of selected Gabor
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
for (i = 1 : nimage)
    Asym{i} = zeros(sx, sy);  
end
tss = zeros(nimage, 1);  % total scores for testing images

binnum = 50; % number of bins 

numCluster = 2; 
%dataWeightAll = zeros(nimage,numCluster);
%for (c = 1:numCluster)
 %   dataWeightAll(floor(rand(1)*nimage)+1, c) = 1.; 
%end
tssAll = rand(nimage, numCluster); 
dataWeightAll = zeros(nimage,numCluster);
symAll = cell(1, numCluster); 
AsymAll = cell(nimage, numCluster); 


% MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mex Clearn.c;   % compile C code

numIt = 5; 
for (it = 1 : numIt)
    %if (it==1) 
     %   Lrange = 0; Orange = 0; 
   % else 
    %    Lrange = 3; Orange = 1; 
   % end
disp(['assign categories in iteration ' num2str(it)]);
for (i = 1 : nimage)
      maxd = max(tssAll(i, :)); 
      for (c = 1 : numCluster) 
       dataWeightAll(i, c) = (tssAll(i, c)==maxd)*1.; 
    end
end

disp(['find the means of categories in iteration ' num2str(it)]);
for (c = 1:numCluster)
    tic
    dataWeight = dataWeightAll(:, c) + 0.; 
    Clearn(nimage, norient, allfiltered, allfilteredback, C, h, sx, sy, Mi, Mx, My, Mm, allsymbol(1, :), sym, Asym, Lrange, Orange, sub, Totalsketch, Upperbound, tss, SHUTUP, binnum, dataWeight);
    disp(['   mex-C learning time for cluster ' num2str(c) ' takes ' num2str(toc) ' seconds']);
    tssAll(:, c) = tss + 0.; 
    symAll{c} = sym + 0.; 
    for (i = 1 : nimage)
        AsymAll{i, c} = Asym{i} + 0.; 
    end
end


if (it==numIt)
 disp(['Displaying the clusters']);
 for (c = 1:numCluster)    
    K = nimage;
    tss = tssAll(:, c); 
    [tsssort, ind] = sort(tss); 
    figure; % display top K examples
    col = 10; row = floor(K/col*2)+1; row = col;
    subplot(row, col, 1, 'v6'); imshow(-(symAll{c}), []); 
    tt = 0; 
    for (i = 1 : K)
       this = ind(nimage-i+1); 
       if (dataWeightAll(this, c)==max(dataWeightAll(this, :)))
          tt = tt + 1; 
          subplot(row, col,  1+tt*2, 'v6'); 
          imshow(I{this}, []);
          subplot(row, col,  2+tt*2, 'v6');
          imshow(-(AsymAll{this, c}), []); 
       end
   end
end
end

end



